@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/cli/cls/cgroup.c    	cgroup classifier module for CLI lib
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2010-2011 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -39,7 +33,7 @@ static void parse_argv(struct rtnl_tc *tc, int argc, char **argv)
 			{ "ematch", 1, 0, 'e' },
 			{ 0, 0, 0, 0 }
 		};
-	
+
 		c = getopt_long(argc, argv, "he:", long_opts, &optidx);
 		if (c == -1)
 			break;
@@ -54,7 +48,7 @@ static void parse_argv(struct rtnl_tc *tc, int argc, char **argv)
 			rtnl_cgroup_set_ematch(cls, tree);
 			break;
 		}
- 	}
+	}
 }
 
 static struct nl_cli_tc_module cgroup_module =
