@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/cls/fw.c		fw classifier
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2013 Thomas Graf <tgraf@suug.ch>
  * Copyright (c) 2006 Petr Gotthard <petr.gotthard@siemens.com>
  * Copyright (c) 2006 Siemens AG Oesterreich
@@ -94,9 +88,12 @@ static int fw_clone(void *_dst, void *_src)
 {
 	struct rtnl_fw *dst = _dst, *src = _src;
 
+	dst->cf_act = NULL;
+	dst->cf_police = NULL;
+
 	if (src->cf_act && !(dst->cf_act = nl_data_clone(src->cf_act)))
 		return -NLE_NOMEM;
-	
+
 	if (src->cf_police && !(dst->cf_police = nl_data_clone(src->cf_police)))
 		return -NLE_NOMEM;
 
