@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/cls/u32.c		u32 classifier
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2013 Thomas Graf <tgraf@suug.ch>
  * Copyright (c) 2005-2006 Petr Gotthard <petr.gotthard@siemens.com>
  * Copyright (c) 2005-2006 Siemens AG Oesterreich
@@ -28,6 +22,8 @@
 #include <netlink/route/cls/u32.h>
 #include <netlink/route/action.h>
 
+#include "netlink-private/utils.h"
+
 /** @cond SKIP */
 #define U32_ATTR_DIVISOR      0x001
 #define U32_ATTR_HASH         0x002
@@ -121,7 +117,7 @@ static int u32_msg_parser(struct rtnl_tc *tc, void *data)
 	if (tb[TCA_U32_ACT]) {
 		u->cu_mask |= U32_ATTR_ACTION;
 		err = rtnl_act_parse(&u->cu_act, tb[TCA_U32_ACT]);
-		if (err)
+		if (err < 0)
 			return err;
 	}
 
@@ -183,27 +179,96 @@ static void u32_free_data(struct rtnl_tc *tc, void *data)
 static int u32_clone(void *_dst, void *_src)
 {
 	struct rtnl_u32 *dst = _dst, *src = _src;
+	_nl_auto_nl_data struct nl_data *selector = NULL;
+	_nl_auto_nl_data struct nl_data *mark = NULL;
+	_nl_auto_nl_data struct nl_data *police = NULL;
+	_nl_auto_nl_data struct nl_data *pcnt = NULL;
+	_nl_auto_nl_data struct nl_data *opts = NULL;
+	_nl_auto_nl_data struct nl_data *xstats = NULL;
+	_nl_auto_nl_data struct nl_data *subdata = NULL;
+	_nl_auto_rtnl_act struct rtnl_act *act = NULL;
 
-	if (src->cu_selector &&
-	    !(dst->cu_selector = nl_data_clone(src->cu_selector)))
-		return -NLE_NOMEM;
+	dst->cu_pcnt = NULL;
+	dst->cu_selector = NULL;
+	dst->cu_mark = NULL;
+	dst->cu_act = NULL;
+	dst->cu_police = NULL;
 
-	if (src->cu_mark &&
-	    !(dst->cu_mark = nl_data_clone(src->cu_mark)))
-		return -NLE_NOMEM;
-
-	if (src->cu_act) {
-		if (!(dst->cu_act = rtnl_act_alloc()))
+	if (src->cu_selector) {
+		if (!(selector = nl_data_clone(src->cu_selector)))
 			return -NLE_NOMEM;
-
-		memcpy(dst->cu_act, src->cu_act, sizeof(struct rtnl_act));
 	}
 
-	if (src->cu_police && !(dst->cu_police = nl_data_clone(src->cu_police)))
-		return -NLE_NOMEM;
+	if (src->cu_mark) {
+		if (!(mark = nl_data_clone(src->cu_mark)))
+			return -NLE_NOMEM;
+	}
 
-	if (src->cu_pcnt && !(dst->cu_pcnt = nl_data_clone(src->cu_pcnt)))
-		return -NLE_NOMEM;
+	if (src->cu_act) {
+		if (!(act = rtnl_act_alloc()))
+			return -NLE_NOMEM;
+
+		if (src->cu_act->c_opts) {
+			if (!(opts = nl_data_clone(src->cu_act->c_opts)))
+				return -NLE_NOMEM;
+		}
+
+		if (src->cu_act->c_xstats) {
+			if (!(xstats = nl_data_clone(src->cu_act->c_xstats)))
+				return -NLE_NOMEM;
+		}
+
+		if (src->cu_act->c_subdata) {
+			if (!(subdata = nl_data_clone(src->cu_act->c_subdata)))
+				return -NLE_NOMEM;
+		}
+	}
+
+	if (src->cu_police) {
+		if (!(police = nl_data_clone(src->cu_police)))
+			return -NLE_NOMEM;
+	}
+
+	if (src->cu_pcnt) {
+		if (!(pcnt = nl_data_clone(src->cu_pcnt)))
+			return -NLE_NOMEM;
+	}
+
+	/* we've passed the critical point and its safe to proceed */
+
+	if (selector)
+		dst->cu_selector = _nl_steal_pointer(&selector);
+
+	if (mark)
+		dst->cu_mark = _nl_steal_pointer(&mark);
+
+	if (police)
+		dst->cu_police = _nl_steal_pointer(&police);
+
+	if (pcnt)
+		dst->cu_pcnt = _nl_steal_pointer(&pcnt);
+
+	if (act) {
+		dst->cu_act = _nl_steal_pointer(&act);
+
+		/* action nl list next and prev pointers must be updated */
+		nl_init_list_head(&dst->cu_act->ce_list);
+
+		if (opts)
+			dst->cu_act->c_opts = _nl_steal_pointer(&opts);
+
+		if (xstats)
+			dst->cu_act->c_xstats = _nl_steal_pointer(&xstats);
+
+		if (subdata)
+			dst->cu_act->c_subdata = _nl_steal_pointer(&subdata);
+
+		if (dst->cu_act->c_link) {
+			nl_object_get(OBJ_CAST(dst->cu_act->c_link));
+		}
+
+		dst->cu_act->a_next = NULL;   /* Only clone first in chain */
+	}
 
 	return 0;
 }
@@ -278,7 +343,9 @@ static void print_selector(struct nl_dump_params *p, struct tc_u32_sel *sel,
 		if (p->dp_type == NL_DUMP_STATS &&
 		    (u->cu_mask & U32_ATTR_PCNT)) {
 			struct tc_u32_pcnt *pcnt = u->cu_pcnt->d_data;
-			nl_dump(p, " successful %" PRIu64, pcnt->kcnts[i]);
+
+			nl_dump(p, " successful %llu",
+				(long long unsigned)pcnt->kcnts[i]);
 		}
 	}
 }
@@ -293,11 +360,6 @@ static void u32_dump_details(struct rtnl_tc *tc, void *data,
 	if (!u)
 		return;
 
-	if (!(u->cu_mask & (U32_ATTR_SELECTOR & U32_ATTR_MARK))) {
-		nl_dump(p, "no-selector no-mark\n");
-		return;
-	}
-
 	if (!(u->cu_mask & U32_ATTR_SELECTOR)) {
 		nl_dump(p, "no-selector");
 	} else {
@@ -338,9 +400,11 @@ static void u32_dump_stats(struct rtnl_tc *tc, void *data,
 
 	if (u->cu_mask & U32_ATTR_PCNT) {
 		struct tc_u32_pcnt *pc = u->cu_pcnt->d_data;
+
 		nl_dump(p, "\n");
-		nl_dump_line(p, "    hit %8" PRIu64 " count %8" PRIu64 "\n",
-			     pc->rhit, pc->rcnt);
+		nl_dump_line(p, "    hit %8llu count %8llu\n",
+			     (long long unsigned)pc->rhit,
+			     (long long unsigned)pc->rcnt);
 	}
 }
 
@@ -373,7 +437,7 @@ static int u32_msg_fill(struct rtnl_tc *tc, void *data, struct nl_msg *msg)
 		int err;
 
 		err = rtnl_act_fill(msg, TCA_U32_ACT, u->cu_act);
-		if (err)
+		if (err < 0)
 			return err;
 	}
 
@@ -526,6 +590,7 @@ int rtnl_u32_set_cls_terminal(struct rtnl_cls *cls)
 int rtnl_u32_add_action(struct rtnl_cls *cls, struct rtnl_act *act)
 {
 	struct rtnl_u32 *u;
+	int err;
 
 	if (!act)
 		return 0;
@@ -534,9 +599,12 @@ int rtnl_u32_add_action(struct rtnl_cls *cls, struct rtnl_act *act)
 		return -NLE_NOMEM;
 
 	u->cu_mask |= U32_ATTR_ACTION;
+	if ((err = rtnl_act_append(&u->cu_act, act)))
+		return err;
+
 	/* In case user frees it */
 	rtnl_act_get(act);
-	return rtnl_act_append(&u->cu_act, act);
+	return 0;
 }
 
 struct rtnl_act* rtnl_u32_get_action(struct rtnl_cls *cls)
