@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/link.c	Links (Interfaces)
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -71,6 +65,7 @@
 #define LINK_ATTR_PHYS_SWITCH_ID	((uint64_t) 1 << 36)
 #define LINK_ATTR_GSO_MAX_SEGS		((uint64_t) 1 << 37)
 #define LINK_ATTR_GSO_MAX_SIZE		((uint64_t) 1 << 38)
+#define LINK_ATTR_LINKINFO_SLAVE_KIND	((uint64_t) 1 << 39)
 
 static struct nl_cache_ops rtnl_link_ops;
 static struct nl_object_ops link_obj_ops;
@@ -91,13 +86,12 @@ static struct rtnl_link_af_ops *af_lookup_and_alloc(struct rtnl_link *link,
 						    int family)
 {
 	struct rtnl_link_af_ops *af_ops;
-	void *data;
 
 	af_ops = rtnl_link_af_ops_lookup(family);
 	if (!af_ops)
 		return NULL;
 
-	if (!(data = rtnl_link_af_alloc(link, af_ops))) {
+	if (!rtnl_link_af_alloc(link, af_ops)) {
 		rtnl_link_af_ops_put(af_ops);
 		return NULL;
 	}
@@ -116,12 +110,12 @@ static int af_free(struct rtnl_link *link, struct rtnl_link_af_ops *ops,
 	return 0;
 }
 
-static int af_request_type(int af_type)
+static int af_request_type(int af_type, struct rtnl_link *changes)
 {
 	struct rtnl_link_af_ops *ops;
 
 	ops = rtnl_link_af_ops_lookup(af_type);
-	if (ops && ops->ao_override_rtm)
+	if (ops && ops->ao_override_rtm(changes))
 		return RTM_SETLINK;
 
 	return RTM_NEWLINK;
@@ -221,22 +215,19 @@ static int af_dump_stats(struct rtnl_link *link, struct rtnl_link_af_ops *ops,
 
 static int do_foreach_af(struct rtnl_link *link,
 			 int (*cb)(struct rtnl_link *,
-			 	   struct rtnl_link_af_ops *, void *, void *),
+				   struct rtnl_link_af_ops *, void *, void *),
 			 void *arg)
 {
 	int i, err;
 
 	for (i = 0; i < AF_MAX; i++) {
 		if (link->l_af_data[i]) {
-			struct rtnl_link_af_ops *ops;
+			_nl_auto_rtnl_link_af_ops struct rtnl_link_af_ops *ops = NULL;
 
 			if (!(ops = rtnl_link_af_ops_lookup(i)))
 				BUG();
 
 			err = cb(link, ops, link->l_af_data[i], arg);
-
-			rtnl_link_af_ops_put(ops);
-
 			if (err < 0)
 				return err;
 		}
@@ -276,6 +267,7 @@ static void link_free_data(struct nl_object *c)
 
 		free(link->l_ifalias);
 		free(link->l_info_kind);
+		free(link->l_info_slave_kind);
 
 		do_foreach_af(link, af_free, NULL);
 
@@ -293,6 +285,19 @@ static int link_clone(struct nl_object *_dst, struct nl_object *_src)
 	struct rtnl_link *src = nl_object_priv(_src);
 	int err;
 
+	dst->l_addr = NULL;
+	dst->l_bcast = NULL;
+	dst->l_info_kind = NULL;
+	dst->l_info_slave_kind = NULL;
+	dst->l_info_ops = NULL;
+	memset(dst->l_af_data, 0, sizeof (dst->l_af_data));
+	dst->l_info = NULL;
+	dst->l_ifalias = NULL;
+	dst->l_af_ops = NULL;
+	dst->l_phys_port_id = NULL;
+	dst->l_phys_switch_id = NULL;
+	dst->l_vf_list = NULL;
+
 	if (src->l_addr)
 		if (!(dst->l_addr = nl_addr_clone(src->l_addr)))
 			return -NLE_NOMEM;
@@ -309,15 +314,28 @@ static int link_clone(struct nl_object *_dst, struct nl_object *_src)
 		if (!(dst->l_info_kind = strdup(src->l_info_kind)))
 			return -NLE_NOMEM;
 
-	if (src->l_info_ops && src->l_info_ops->io_clone) {
-		err = src->l_info_ops->io_clone(dst, src);
-		if (err < 0)
-			return err;
+	if (src->l_info_slave_kind)
+		if (!(dst->l_info_slave_kind = strdup(src->l_info_slave_kind)))
+			return -NLE_NOMEM;
+
+	if (src->l_info_ops) {
+
+		rtnl_link_info_ops_get(src->l_info_ops);
+		dst->l_info_ops = src->l_info_ops;
+
+		if (src->l_info_ops->io_clone) {
+			err = src->l_info_ops->io_clone(dst, src);
+			if (err < 0)
+				return err;
+		}
 	}
 
 	if ((err = do_foreach_af(src, af_clone, dst)) < 0)
 		return err;
 
+	if (src->l_af_ops)
+		dst->l_af_ops = af_lookup_and_alloc(dst, src->l_af_ops->ao_family);
+
 	if (src->l_phys_port_id)
 		if (!(dst->l_phys_port_id = nl_data_clone(src->l_phys_port_id)))
 			return -NLE_NOMEM;
@@ -380,7 +398,7 @@ int rtnl_link_info_parse(struct rtnl_link *link, struct nlattr **tb)
 		return -NLE_MISSING_ATTR;
 
 	nla_strlcpy(link->l_name, tb[IFLA_IFNAME], IFNAMSIZ);
-
+	link->ce_mask |= LINK_ATTR_IFNAME;
 
 	if (tb[IFLA_STATS]) {
 		struct rtnl_link_stats *st = nla_data(tb[IFLA_STATS]);
@@ -520,7 +538,7 @@ int rtnl_link_info_parse(struct rtnl_link *link, struct nlattr **tb)
 	}
 
 	if (tb[IFLA_MAP]) {
-		nla_memcpy(&link->l_map, tb[IFLA_MAP], 
+		nla_memcpy(&link->l_map, tb[IFLA_MAP],
 			   sizeof(struct rtnl_link_ifmap));
 		link->ce_mask |= LINK_ATTR_MAP;
 	}
@@ -573,28 +591,22 @@ int rtnl_link_info_parse(struct rtnl_link *link, struct nlattr **tb)
 static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 			   struct nlmsghdr *n, struct nl_parser_param *pp)
 {
-	struct rtnl_link *link;
+	_nl_auto_rtnl_link struct rtnl_link *link = NULL;
+	struct nla_policy real_link_policy[ARRAY_SIZE(rtln_link_policy)];
+	struct nla_policy *link_policy = rtln_link_policy;
+	struct rtnl_link_af_ops *af_ops_family;
 	struct ifinfomsg *ifi;
 	struct nlattr *tb[IFLA_MAX+1];
-	struct rtnl_link_af_ops *af_ops = NULL;
-	struct rtnl_link_af_ops *af_ops_family;
 	int err, family;
-	struct nla_policy real_link_policy[IFLA_MAX+1];
-
-	memcpy(&real_link_policy, rtln_link_policy, sizeof(rtln_link_policy));
 
 	link = rtnl_link_alloc();
-	if (link == NULL) {
-		err = -NLE_NOMEM;
-		goto errout;
-	}
+	if (link == NULL)
+		return -NLE_NOMEM;
 
 	link->ce_msgtype = n->nlmsg_type;
 
-	if (!nlmsg_valid_hdr(n, sizeof(*ifi))) {
-		err = -NLE_MSG_TOOSHORT;
-		goto errout;
-	}
+	if (!nlmsg_valid_hdr(n, sizeof(*ifi)))
+		return -NLE_MSG_TOOSHORT;
 
 	ifi = nlmsg_data(n);
 	link->l_family = family = ifi->ifi_family;
@@ -602,35 +614,37 @@ static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 	link->l_index = ifi->ifi_index;
 	link->l_flags = ifi->ifi_flags;
 	link->l_change = ifi->ifi_change;
-	link->ce_mask = (LINK_ATTR_IFNAME | LINK_ATTR_FAMILY |
+	link->ce_mask = (LINK_ATTR_FAMILY |
 			 LINK_ATTR_ARPTYPE| LINK_ATTR_IFINDEX |
 			 LINK_ATTR_FLAGS | LINK_ATTR_CHANGE);
 
-	if ((af_ops_family = af_ops = af_lookup_and_alloc(link, family))) {
-		if (af_ops->ao_protinfo_policy) {
+	if ((link->l_af_ops = af_lookup_and_alloc(link, family))) {
+		if (link->l_af_ops->ao_protinfo_policy) {
+			_NL_STATIC_ASSERT (sizeof(rtln_link_policy) == sizeof(real_link_policy));
+			memcpy(&real_link_policy, rtln_link_policy, sizeof(rtln_link_policy));
 			memcpy(&real_link_policy[IFLA_PROTINFO],
-			       af_ops->ao_protinfo_policy,
+			       link->l_af_ops->ao_protinfo_policy,
 			       sizeof(struct nla_policy));
+			link_policy = real_link_policy;
 		}
-
-		link->l_af_ops = af_ops;
 	}
 
-	err = nlmsg_parse(n, sizeof(*ifi), tb, IFLA_MAX, real_link_policy);
+	af_ops_family = link->l_af_ops;
+
+	err = nlmsg_parse(n, sizeof(*ifi), tb, IFLA_MAX, link_policy);
 	if (err < 0)
-		goto errout;
+		return err;
 
 	err = rtnl_link_info_parse(link, tb);
 	if (err < 0)
-		goto errout;
+		return err;
 
 	if (tb[IFLA_NUM_VF]) {
 		link->l_num_vf = nla_get_u32(tb[IFLA_NUM_VF]);
 		link->ce_mask |= LINK_ATTR_NUM_VF;
 		if (link->l_num_vf && tb[IFLA_VFINFO_LIST]) {
-			if ((err = rtnl_link_sriov_parse_vflist(link, tb)) < 0) {
-				goto errout;
-			}
+			if ((err = rtnl_link_sriov_parse_vflist(link, tb)) < 0)
+				return err;
 			link->ce_mask |= LINK_ATTR_VF_LIST;
 		}
 	}
@@ -641,27 +655,28 @@ static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 		err = nla_parse_nested(li, IFLA_INFO_MAX, tb[IFLA_LINKINFO],
 				       link_info_policy);
 		if (err < 0)
-			goto errout;
+			return err;
 
 		if (li[IFLA_INFO_KIND]) {
 			struct rtnl_link_info_ops *ops;
-			char *kind = nla_get_string(li[IFLA_INFO_KIND]);
+			const char *kind = nla_get_string(li[IFLA_INFO_KIND]);
 			int af;
 
 			err = rtnl_link_set_type(link, kind);
 			if (err < 0)
-				goto errout;
+				return err;
 
-			if ((af = nl_str2af(kind)) >= 0 &&
-				!af_ops && (af_ops = af_lookup_and_alloc(link, af))) {
-
-				if (af_ops->ao_protinfo_policy) {
-					tb[IFLA_PROTINFO] = (struct nlattr *)af_ops->ao_protinfo_policy;
-				}
+			if (   (af = nl_str2af(kind)) >= 0
+			    && !link->l_af_ops
+			    && (link->l_af_ops = af_lookup_and_alloc(link, af))) {
 				link->l_family = af;
-				link->l_af_ops = af_ops;
+				if (link->l_af_ops->ao_protinfo_policy)
+					tb[IFLA_PROTINFO] = (struct nlattr *)link->l_af_ops->ao_protinfo_policy;
 			}
 
+			if (link->l_info_ops)
+				release_link_info(link);
+
 			ops = rtnl_link_info_ops_lookup(kind);
 			link->l_info_ops = ops;
 
@@ -671,20 +686,33 @@ static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 					err = ops->io_parse(link, li[IFLA_INFO_DATA],
 							    li[IFLA_INFO_XSTATS]);
 					if (err < 0)
-						goto errout;
+						return err;
 				} else {
 					/* XXX: Warn about unparsed info? */
 				}
 			}
+
+			link->ce_mask |= LINK_ATTR_LINKINFO;
+		}
+
+		if (li[IFLA_INFO_SLAVE_KIND]) {
+			const char *kind = nla_get_string(li[IFLA_INFO_SLAVE_KIND]);
+
+			err = rtnl_link_set_slave_type(link, kind);
+			if (err < 0)
+				return err;
+
+			link->ce_mask |= LINK_ATTR_LINKINFO_SLAVE_KIND;
 		}
-		link->ce_mask |= LINK_ATTR_LINKINFO;
 	}
 
-	if (tb[IFLA_PROTINFO] && af_ops && af_ops->ao_parse_protinfo) {
-		err = af_ops->ao_parse_protinfo(link, tb[IFLA_PROTINFO],
-						link->l_af_data[link->l_family]);
+	if (   tb[IFLA_PROTINFO]
+	    && link->l_af_ops
+	    && link->l_af_ops->ao_parse_protinfo) {
+		err = link->l_af_ops->ao_parse_protinfo(link, tb[IFLA_PROTINFO],
+		                                        link->l_af_data[link->l_family]);
 		if (err < 0)
-			goto errout;
+			return err;
 		link->ce_mask |= LINK_ATTR_PROTINFO;
 	}
 
@@ -692,25 +720,28 @@ static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 		/* parsing of IFLA_AF_SPEC is dependent on the family used
 		 * in the request message.
 		 */
-		if (af_ops_family && af_ops_family->ao_parse_af_full) {
+		if (   af_ops_family
+		    && af_ops_family->ao_parse_af_full) {
 			err = af_ops_family->ao_parse_af_full(link,
 			                                      tb[IFLA_AF_SPEC],
 			                                      link->l_af_data[af_ops_family->ao_family]);
 			if (err < 0)
-				goto errout;
+				return err;
 			link->ce_mask |= LINK_ATTR_AF_SPEC;
 		} else if (family == AF_UNSPEC) {
 			struct nlattr *af_attr;
 			int remaining;
 
 			nla_for_each_nested(af_attr, tb[IFLA_AF_SPEC], remaining) {
+				_nl_auto_rtnl_link_af_ops struct rtnl_link_af_ops *af_ops = NULL;
+
 				af_ops = af_lookup_and_alloc(link, nla_type(af_attr));
 				if (af_ops && af_ops->ao_parse_af) {
 					char *af_data = link->l_af_data[nla_type(af_attr)];
 
 					err = af_ops->ao_parse_af(link, af_attr, af_data);
 					if (err < 0)
-						goto errout;
+						return err;
 				}
 			}
 			link->ce_mask |= LINK_ATTR_AF_SPEC;
@@ -752,10 +783,8 @@ static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 
 	if (tb[IFLA_PHYS_PORT_ID]) {
 		link->l_phys_port_id = nl_data_alloc_attr(tb[IFLA_PHYS_PORT_ID]);
-		if (link->l_phys_port_id == NULL) {
-			err = -NLE_NOMEM;
-			goto errout;
-		}
+		if (link->l_phys_port_id == NULL)
+			return -NLE_NOMEM;
 		link->ce_mask |= LINK_ATTR_PHYS_PORT_ID;
 	}
 
@@ -766,26 +795,20 @@ static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 
 	if (tb[IFLA_PHYS_SWITCH_ID]) {
 		link->l_phys_switch_id = nl_data_alloc_attr(tb[IFLA_PHYS_SWITCH_ID]);
-		if (link->l_phys_switch_id == NULL) {
-			err = -NLE_NOMEM;
-			goto errout;
-		}
+		if (link->l_phys_switch_id == NULL)
+			return -NLE_NOMEM;
 		link->ce_mask |= LINK_ATTR_PHYS_SWITCH_ID;
 	}
 
-	err = pp->pp_cb((struct nl_object *) link, pp);
-errout:
-	rtnl_link_af_ops_put(af_ops);
-	rtnl_link_put(link);
-	return err;
+	return pp->pp_cb((struct nl_object *) link, pp);
 }
 
 static int link_request_update(struct nl_cache *cache, struct nl_sock *sk)
 {
+	_nl_auto_nl_msg struct nl_msg *msg = NULL;
 	int family = cache->c_iarg1;
 	struct ifinfomsg hdr = { .ifi_family = family };
 	struct rtnl_link_af_ops *ops;
-	struct nl_msg *msg;
 	int err;
 	__u32 ext_filter_mask = RTEXT_FILTER_VF;
 
@@ -793,30 +816,27 @@ static int link_request_update(struct nl_cache *cache, struct nl_sock *sk)
 	if (!msg)
 		return -NLE_NOMEM;
 
-	err = -NLE_MSGSIZE;
 	if (nlmsg_append(msg, &hdr, sizeof(hdr), NLMSG_ALIGNTO) < 0)
-		goto nla_put_failure;
+		return -NLE_MSGSIZE;
 
 	ops = rtnl_link_af_ops_lookup(family);
 	if (ops && ops->ao_get_af) {
 		err = ops->ao_get_af(msg, &ext_filter_mask);
-		if (err)
-			goto nla_put_failure;
+		if (err < 0)
+			return err;
 	}
 
 	if (ext_filter_mask) {
 		err = nla_put(msg, IFLA_EXT_MASK, sizeof(ext_filter_mask), &ext_filter_mask);
-		if (err)
-			goto nla_put_failure;
+		if (err < 0)
+			return err;
 	}
 
 	err = nl_send_auto(sk, msg);
-	if (err > 0)
-		err = 0;
+	if (err < 0)
+		return 0;
 
-nla_put_failure:
-	nlmsg_free(msg);
-	return err;
+	return 0;
 }
 
 static void link_dump_line(struct nl_object *obj, struct nl_dump_params *p)
@@ -831,6 +851,9 @@ static void link_dump_line(struct nl_object *obj, struct nl_dump_params *p)
 		fetched_cache = 1;
 	}
 
+	if (link->l_family != AF_UNSPEC)
+		nl_dump_line(p, "%s ", nl_af2str(link->l_family, buf, sizeof(buf)));
+
 	nl_dump_line(p, "%s %s ", link->l_name,
 		     nl_llproto2str(link->l_arptype, buf, sizeof(buf)));
 
@@ -839,10 +862,9 @@ static void link_dump_line(struct nl_object *obj, struct nl_dump_params *p)
 
 	if (link->ce_mask & LINK_ATTR_MASTER) {
 		if (cache) {
-			struct rtnl_link *master = rtnl_link_get(cache, link->l_master);
+			_nl_auto_rtnl_link struct rtnl_link *master = rtnl_link_get(cache, link->l_master);
+
 			nl_dump(p, "master %s ", master ? master->l_name : "inv");
-			if (master)
-				rtnl_link_put(master);
 		} else
 			nl_dump(p, "master %d ", link->l_master);
 	}
@@ -854,10 +876,9 @@ static void link_dump_line(struct nl_object *obj, struct nl_dump_params *p)
 	if (link->ce_mask & LINK_ATTR_LINK) {
 		if (   cache
 		    && !(link->ce_mask & LINK_ATTR_LINK_NETNSID)) {
-			struct rtnl_link *ll = rtnl_link_get(cache, link->l_link);
+			_nl_auto_rtnl_link struct rtnl_link *ll = rtnl_link_get(cache, link->l_link);
+
 			nl_dump(p, "slave-of %s ", ll ? ll->l_name : "NONE");
-			if (ll)
-				rtnl_link_put(ll);
 		} else
 			nl_dump(p, "slave-of %d ", link->l_link);
 	}
@@ -1348,10 +1369,9 @@ struct rtnl_link *rtnl_link_get_by_name(struct nl_cache *cache,
 int rtnl_link_build_get_request(int ifindex, const char *name,
 				struct nl_msg **result)
 {
+	_nl_auto_nl_msg struct nl_msg *msg = NULL;
 	struct ifinfomsg ifi;
-	struct nl_msg *msg;
 	__u32 vf_mask = RTEXT_FILTER_VF;
-	int err = -NLE_MSGSIZE;
 
 	if (ifindex <= 0 && !name) {
 		APPBUG("ifindex or name must be specified");
@@ -1366,24 +1386,15 @@ int rtnl_link_build_get_request(int ifindex, const char *name,
 	if (ifindex > 0)
 		ifi.ifi_index = ifindex;
 
-	if (nlmsg_append(msg, &ifi, sizeof(ifi), NLMSG_ALIGNTO) < 0) {
-		err = -NLE_MSGSIZE;
-		goto nla_put_failure;
-	}
+	_NL_RETURN_ON_PUT_ERR(nlmsg_append(msg, &ifi, sizeof(ifi), NLMSG_ALIGNTO));
 
 	if (name)
-		NLA_PUT_STRING(msg, IFLA_IFNAME, name);
+		_NL_RETURN_ON_PUT_ERR(nla_put_string(msg, IFLA_IFNAME, name));
 
-	err = nla_put(msg, IFLA_EXT_MASK, sizeof(vf_mask), &vf_mask);
-	if (err)
-		goto nla_put_failure;
+	_NL_RETURN_ON_PUT_ERR(nla_put(msg, IFLA_EXT_MASK, sizeof(vf_mask), &vf_mask));
 
-	*result = msg;
+	*result = _nl_steal_pointer(&msg);
 	return 0;
-
-nla_put_failure:
-	nlmsg_free(msg);
-	return err;
 }
 
 /**
@@ -1410,8 +1421,8 @@ nla_put_failure:
 int rtnl_link_get_kernel(struct nl_sock *sk, int ifindex, const char *name,
 			 struct rtnl_link **result)
 {
-	struct nl_msg *msg = NULL;
-	struct nl_object *obj;
+	_nl_auto_rtnl_link struct rtnl_link *link = NULL;
+	_nl_auto_nl_msg struct nl_msg *msg = NULL;
 	int err;
 	int syserr;
 
@@ -1419,14 +1430,15 @@ int rtnl_link_get_kernel(struct nl_sock *sk, int ifindex, const char *name,
 		return err;
 
 	err = nl_send_auto(sk, msg);
-	nlmsg_free(msg);
 	if (err < 0)
 		return err;
 
-	if ((err = nl_pickup_keep_syserr(sk, link_msg_parser, &obj, &syserr)) < 0) {
-		if (syserr == -EINVAL &&
-		    ifindex <= 0 &&
-		    name && *name) {
+	err = nl_pickup_keep_syserr(sk, link_msg_parser, (struct nl_object **) &link, &syserr);
+	if (err < 0) {
+		if (   syserr == -EINVAL
+		    && ifindex <= 0
+		    && name
+		    && *name) {
 			/* Older kernels do not support lookup by ifname. This was added
 			 * by commit kernel a3d1289126e7b14307074b76bf1677015ea5036f .
 			 * Detect this error case and return NLE_OPNOTSUPP instead of
@@ -1436,13 +1448,11 @@ int rtnl_link_get_kernel(struct nl_sock *sk, int ifindex, const char *name,
 		return err;
 	}
 
-	/* We have used link_msg_parser(), object is definitely a link */
-	*result = (struct rtnl_link *) obj;
-
 	/* If an object has been returned, we also need to wait for the ACK */
-	if (err == 0 && obj)
+	if (err == 0 && link)
 		wait_for_ack(sk);
 
+	*result = _nl_steal_pointer(&link);
 	return 0;
 }
 
@@ -1463,11 +1473,11 @@ int rtnl_link_get_kernel(struct nl_sock *sk, int ifindex, const char *name,
 char * rtnl_link_i2name(struct nl_cache *cache, int ifindex, char *dst,
 			size_t len)
 {
-	struct rtnl_link *link = rtnl_link_get(cache, ifindex);
+	_nl_auto_rtnl_link struct rtnl_link *link = NULL;
 
+	link = rtnl_link_get(cache, ifindex);
 	if (link) {
-		strncpy(dst, link->l_name, len - 1);
-		rtnl_link_put(link);
+		_nl_strncpy_trunc(dst, link->l_name, len);
 		return dst;
 	}
 
@@ -1485,16 +1495,13 @@ char * rtnl_link_i2name(struct nl_cache *cache, int ifindex, char *dst,
  */
 int rtnl_link_name2i(struct nl_cache *cache, const char *name)
 {
-	int ifindex = 0;
-	struct rtnl_link *link;
+	_nl_auto_rtnl_link struct rtnl_link *link = NULL;
 
 	link = rtnl_link_get_by_name(cache, name);
-	if (link) {
-		ifindex = link->l_index;
-		rtnl_link_put(link);
-	}
+	if (link)
+		return link->l_index;
 
-	return ifindex;
+	return 0;
 }
 
 /** @} */
@@ -1561,7 +1568,7 @@ nla_put_failure:
 static int build_link_msg(int cmd, struct ifinfomsg *hdr,
 			  struct rtnl_link *link, int flags, struct nl_msg **result)
 {
-	struct nl_msg *msg;
+	_nl_auto_nl_msg struct nl_msg *msg = NULL;
 	struct nlattr *af_spec;
 
 	msg = nlmsg_alloc_simple(cmd, flags);
@@ -1577,18 +1584,24 @@ static int build_link_msg(int cmd, struct ifinfomsg *hdr,
 	if (link->ce_mask & LINK_ATTR_GROUP)
 		NLA_PUT_U32(msg, IFLA_GROUP, link->l_group);
 
-	if (link->ce_mask & LINK_ATTR_LINKINFO) {
+	if (link->ce_mask & (LINK_ATTR_LINKINFO|LINK_ATTR_LINKINFO_SLAVE_KIND)) {
 		struct nlattr *info;
 
 		if (!(info = nla_nest_start(msg, IFLA_LINKINFO)))
 			goto nla_put_failure;
 
-		NLA_PUT_STRING(msg, IFLA_INFO_KIND, link->l_info_kind);
+		if (link->ce_mask & LINK_ATTR_LINKINFO) {
+			NLA_PUT_STRING(msg, IFLA_INFO_KIND, link->l_info_kind);
 
-		if (link->l_info_ops) {
-			if (link->l_info_ops->io_put_attrs &&
-			    link->l_info_ops->io_put_attrs(msg, link) < 0)
-				goto nla_put_failure;
+			if (link->l_info_ops) {
+				if (link->l_info_ops->io_put_attrs &&
+				    link->l_info_ops->io_put_attrs(msg, link) < 0)
+					goto nla_put_failure;
+			}
+		}
+
+		if (link->ce_mask & LINK_ATTR_LINKINFO_SLAVE_KIND) {
+			NLA_PUT_STRING(msg, IFLA_INFO_SLAVE_KIND, link->l_info_slave_kind);
 		}
 
 		nla_nest_end(msg, info);
@@ -1610,11 +1623,10 @@ static int build_link_msg(int cmd, struct ifinfomsg *hdr,
 
 	nla_nest_end(msg, af_spec);
 
-	*result = msg;
+	*result = _nl_steal_pointer(&msg);
 	return 0;
 
 nla_put_failure:
-	nlmsg_free(msg);
 	return -NLE_MSGSIZE;
 }
 
@@ -1729,15 +1741,12 @@ int rtnl_link_build_change_request(struct rtnl_link *orig,
 	    !strcmp(orig->l_name, changes->l_name))
 		changes->ce_mask &= ~LINK_ATTR_IFNAME;
 
-	rt = af_request_type(orig->l_family);
+	rt = af_request_type(orig->l_family, changes);
 
 	if ((err = build_link_msg(rt, &ifi, changes, flags, result)) < 0)
-		goto errout;
+		return err;
 
 	return 0;
-
-errout:
-	return err;
 }
 
 /**
@@ -1769,32 +1778,40 @@ errout:
  * @note The link name can only be changed if the link has been put
  *       in opertional down state. (~IF_UP)
  *
+ * @note On versions up to 3.4.0, \c NLE_SEQ_MISMATCH would be returned if the
+ *       kernel does not supports \c RTM_NEWLINK. It is advised to ignore the
+ *       error code if you cannot upgrade the library.
+ *
  * @return 0 on success or a negative error code.
  */
 int rtnl_link_change(struct nl_sock *sk, struct rtnl_link *orig,
 		     struct rtnl_link *changes, int flags)
 {
-	struct nl_msg *msg;
+	_nl_auto_nl_msg struct nl_msg *msg = NULL;
 	int err;
 
 	err = rtnl_link_build_change_request(orig, changes, flags, &msg);
 	if (err < 0)
 		return err;
 
+	BUG_ON(msg->nm_nlh->nlmsg_seq != NL_AUTO_SEQ);
 retry:
 	err = nl_send_auto_complete(sk, msg);
 	if (err < 0)
-		goto errout;
+		return err;
 
 	err = wait_for_ack(sk);
-	if (err == -NLE_OPNOTSUPP && msg->nm_nlh->nlmsg_type == RTM_NEWLINK) {
+	if (   err == -NLE_OPNOTSUPP
+	    && msg->nm_nlh->nlmsg_type == RTM_NEWLINK) {
 		msg->nm_nlh->nlmsg_type = RTM_SETLINK;
+		msg->nm_nlh->nlmsg_seq = NL_AUTO_SEQ;
 		goto retry;
 	}
 
-errout:
-	nlmsg_free(msg);
-	return err;
+	if (err < 0)
+		return err;
+
+	return 0;
 }
 
 /** @} */
@@ -1820,7 +1837,7 @@ errout:
 int rtnl_link_build_delete_request(const struct rtnl_link *link,
 				   struct nl_msg **result)
 {
-	struct nl_msg *msg;
+	_nl_auto_nl_msg struct nl_msg *msg = NULL;
 	struct ifinfomsg ifi = {
 		.ifi_index = link->l_index,
 	};
@@ -1833,18 +1850,13 @@ int rtnl_link_build_delete_request(const struct rtnl_link *link,
 	if (!(msg = nlmsg_alloc_simple(RTM_DELLINK, 0)))
 		return -NLE_NOMEM;
 
-	if (nlmsg_append(msg, &ifi, sizeof(ifi), NLMSG_ALIGNTO) < 0)
-		goto nla_put_failure;
+	_NL_RETURN_ON_PUT_ERR(nlmsg_append(msg, &ifi, sizeof(ifi), NLMSG_ALIGNTO));
 
 	if (link->ce_mask & LINK_ATTR_IFNAME)
-		NLA_PUT_STRING(msg, IFLA_IFNAME, link->l_name);
+		_NL_RETURN_ON_PUT_ERR(nla_put_string(msg, IFLA_IFNAME, link->l_name));
 
-	*result = msg;
+	*result = _nl_steal_pointer(&msg);
 	return 0;
-
-nla_put_failure:
-	nlmsg_free(msg);
-	return -NLE_MSGSIZE;
 }
 
 /**
@@ -1925,7 +1937,7 @@ void rtnl_link_put(struct rtnl_link *link)
  */
 void rtnl_link_set_name(struct rtnl_link *link, const char *name)
 {
-	strncpy(link->l_name, name, sizeof(link->l_name) - 1);
+	_nl_strncpy_trunc(link->l_name, name, sizeof(link->l_name));
 	link->ce_mask |= LINK_ATTR_IFNAME;
 }
 
@@ -2092,9 +2104,10 @@ void rtnl_link_set_family(struct rtnl_link *link, int family)
 	link->ce_mask |= LINK_ATTR_FAMILY;
 
 	if (link->l_af_ops) {
-		af_free(link, link->l_af_ops,
-			link->l_af_data[link->l_af_ops->ao_family], NULL);
-		link->l_af_data[link->l_af_ops->ao_family] = NULL;
+		int ao_family = link->l_af_ops->ao_family;
+
+		af_free(link, link->l_af_ops, link->l_af_data[ao_family], NULL);
+		link->l_af_data[ao_family] = NULL;
 	}
 
 	link->l_af_ops = af_lookup_and_alloc(link, family);
@@ -2449,7 +2462,7 @@ void rtnl_link_set_ifalias(struct rtnl_link *link, const char *alias)
  */
 void rtnl_link_set_qdisc(struct rtnl_link *link, const char *name)
 {
-	strncpy(link->l_qdisc, name, sizeof(link->l_qdisc) - 1);
+	_nl_strncpy_trunc(link->l_qdisc, name, sizeof(link->l_qdisc));
 	link->ce_mask |= LINK_ATTR_QDISC;
 }
 
@@ -2530,13 +2543,13 @@ int rtnl_link_set_stat(struct rtnl_link *link, rtnl_link_stat_id_t id,
  * be released with all link type specific attributes lost.
  *
  * @route_doc{link_modules, Link Modules}
- * @return 0 on success or a negative errror code.
+ * @return 0 on success or a negative error code.
  */
 int rtnl_link_set_type(struct rtnl_link *link, const char *type)
 {
 	struct rtnl_link_info_ops *io;
+	_nl_auto_free char *kind = NULL;
 	int err;
-	char *kind;
 
 	free(link->l_info_kind);
 	link->ce_mask &= ~LINK_ATTR_LINKINFO;
@@ -2551,20 +2564,18 @@ int rtnl_link_set_type(struct rtnl_link *link, const char *type)
 
 	io = rtnl_link_info_ops_lookup(type);
 	if (io) {
-		if (io->io_alloc && (err = io->io_alloc(link)) < 0)
-			goto errout;
+		if (io->io_alloc && (err = io->io_alloc(link)) < 0) {
+			_nl_clear_free(&kind);
+			return err;
+		}
 
 		link->l_info_ops = io;
 	}
 
-	link->l_info_kind = kind;
+	link->l_info_kind = _nl_steal_pointer(&kind);
 	link->ce_mask |= LINK_ATTR_LINKINFO;
 
 	return 0;
-
-errout:
-	free(kind);
-	return err;
 }
 
 /**
@@ -2579,6 +2590,49 @@ char *rtnl_link_get_type(struct rtnl_link *link)
 	return link->l_info_kind;
 }
 
+/**
+ * Set type of slave link object
+ * @arg link		Link object (slave)
+ * @arg type		Name of link type
+ *
+ * If a slave type has been assigned already it will be released.
+ *
+ * @route_doc{link_modules, Link Modules}
+ * @return 0 on success or a negative error code.
+ */
+int rtnl_link_set_slave_type(struct rtnl_link *link, const char *type)
+{
+	char *kind = NULL;
+
+	if (type) {
+		kind = strdup(type);
+		if (!kind)
+			return -NLE_NOMEM;
+	}
+
+	free(link->l_info_slave_kind);
+	link->l_info_slave_kind = kind;
+
+	if (kind)
+		link->ce_mask |= LINK_ATTR_LINKINFO_SLAVE_KIND;
+	else
+		link->ce_mask &= ~LINK_ATTR_LINKINFO_SLAVE_KIND;
+	return 0;
+}
+
+/**
+ * Return type of enslaved link
+ * @arg link		Link object
+ *
+ * @route_doc{link_modules, Link Modules}
+ * @return Name of enslaved link type or NULL if not specified.
+ */
+const char *rtnl_link_get_slave_type(const struct rtnl_link *link)
+{
+	return link->l_info_slave_kind;
+}
+
+
 /**
  * Set link promiscuity count
  * @arg link		Link object
@@ -2781,7 +2835,7 @@ pid_t rtnl_link_get_ns_pid(struct rtnl_link *link)
  */
 int rtnl_link_enslave_ifindex(struct nl_sock *sock, int master, int slave)
 {
-	struct rtnl_link *link;
+	_nl_auto_rtnl_link struct rtnl_link *link = NULL;
 	int err;
 
 	if (!(link = rtnl_link_alloc()))
@@ -2791,12 +2845,12 @@ int rtnl_link_enslave_ifindex(struct nl_sock *sock, int master, int slave)
 	rtnl_link_set_master(link, master);
 
 	if ((err = rtnl_link_change(sock, link, link, 0)) < 0)
-		goto errout;
+		return err;
 
-	rtnl_link_put(link);
+	_nl_clear_pointer(&link, rtnl_link_put);
 
 	/*
-	 * Due to the kernel not signaling whether this opertion is
+	 * Due to the kernel not signaling whether this operation is
 	 * supported or not, we will retrieve the attribute to see  if the
 	 * request was successful. If the master assigned remains unchanged
 	 * we will return NLE_OPNOTSUPP to allow performing backwards
@@ -2806,12 +2860,9 @@ int rtnl_link_enslave_ifindex(struct nl_sock *sock, int master, int slave)
 		return err;
 
 	if (rtnl_link_get_master(link) != master)
-		err = -NLE_OPNOTSUPP;
+		return -NLE_OPNOTSUPP;
 
-errout:
-	rtnl_link_put(link);
-
-	return err;
+	return 0;
 }
 
 /**
@@ -2987,6 +3038,7 @@ static const struct trans_tbl link_stats[] = {
 	__ADD(RTNL_LINK_IP6_ECT0PKTS, Ip6_InECT0Pkts),
 	__ADD(RTNL_LINK_IP6_CEPKTS, Ip6_InCEPkts),
 	__ADD(RTNL_LINK_RX_NOHANDLER, rx_nohandler),
+	__ADD(RTNL_LINK_REASM_OVERLAPS, ReasmOverlaps),
 };
 
 char *rtnl_link_stat2str(int st, char *buf, size_t len)
@@ -3027,8 +3079,8 @@ static const struct trans_tbl link_modes[] = {
 };
 
 static const struct trans_tbl carrier_states[] = {
-	__ADD(IF_CARRIER_DOWN, down),
-	__ADD(IF_CARRIER_UP, up),
+	__ADD(0, down),
+	__ADD(1, up),
 };
 
 char *rtnl_link_mode2str(uint8_t st, char *buf, size_t len)
@@ -3059,22 +3111,16 @@ int rtnl_link_has_vf_list(struct rtnl_link *link) {
 		return 0;
 }
 
-void rtnl_link_set_vf_list(struct rtnl_link *link) {
-	int err;
-
-	if (!(err = rtnl_link_has_vf_list(link)))
+void rtnl_link_set_vf_list(struct rtnl_link *link)
+{
+	if (!rtnl_link_has_vf_list(link))
 		link->ce_mask |= LINK_ATTR_VF_LIST;
-
-	return;
 }
 
-void rtnl_link_unset_vf_list(struct rtnl_link *link) {
-	int err;
-
-	if ((err = rtnl_link_has_vf_list(link)))
+void rtnl_link_unset_vf_list(struct rtnl_link *link)
+{
+	if (rtnl_link_has_vf_list(link))
 		link->ce_mask &= ~LINK_ATTR_VF_LIST;
-
-	return;
 }
 
 /** @} */
@@ -3137,6 +3183,7 @@ static struct nl_object_ops link_obj_ops = {
 static struct nl_af_group link_groups[] = {
 	{ AF_UNSPEC,	RTNLGRP_LINK },
 	{ AF_BRIDGE,    RTNLGRP_LINK },
+	{ AF_INET6,     RTNLGRP_IPV6_IFINFO },
 	{ END_OF_GROUP_LIST },
 };
 
