@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/link/api.c		Link Info API
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -86,14 +80,33 @@ struct rtnl_link_info_ops *rtnl_link_info_ops_lookup(const char *name)
 	return ops;
 }
 
+/**
+ * Take reference to a set of operations.
+ * @arg ops		Link info operations.
+ */
+void rtnl_link_info_ops_get(struct rtnl_link_info_ops *ops)
+{
+	if (!ops)
+		return;
+
+	nl_write_lock(&info_lock);
+	ops->io_refcnt++;
+	nl_write_unlock(&info_lock);
+}
+
 /**
  * Give back reference to a set of operations.
  * @arg ops		Link info operations.
  */
 void rtnl_link_info_ops_put(struct rtnl_link_info_ops *ops)
 {
-	if (ops)
-		ops->io_refcnt--;
+	if (!ops)
+		return;
+
+	nl_write_lock(&info_lock);
+	_nl_assert(ops->io_refcnt > 0);
+	ops->io_refcnt--;
+	nl_write_unlock(&info_lock);
 }
 
 /**
@@ -152,6 +165,7 @@ int rtnl_link_unregister_info(struct rtnl_link_info_ops *ops)
 
 	nl_list_for_each_entry(t, &info_ops, io_list) {
 		if (t == ops) {
+			_nl_assert(t->io_refcnt >= 0);
 			if (t->io_refcnt > 0) {
 				err = -NLE_BUSY;
 				goto errout;
@@ -208,8 +222,11 @@ struct rtnl_link_af_ops *rtnl_link_af_ops_lookup(const unsigned int family)
  */
 void rtnl_link_af_ops_put(struct rtnl_link_af_ops *ops)
 {
-	if (ops)
+	if (ops) {
+		nl_write_lock(&info_lock);
 		ops->ao_refcnt--;
+		nl_write_unlock(&info_lock);
+	}
 }
 
 /**
@@ -326,7 +343,7 @@ int rtnl_link_af_unregister(struct rtnl_link_af_ops *ops)
 	int err = -NLE_INVAL;
 
 	if (!ops)
-		goto errout;
+		return err;
 
 	nl_write_lock(&info_lock);
 	if (!af_ops[ops->ao_family]) {
@@ -345,7 +362,7 @@ int rtnl_link_af_unregister(struct rtnl_link_af_ops *ops)
 		ops->ao_family);
 
 errout:
-	nl_write_lock(&info_lock);
+	nl_write_unlock(&info_lock);
 
 	return err;
 }
@@ -365,28 +382,54 @@ errout:
 int rtnl_link_af_data_compare(struct rtnl_link *a, struct rtnl_link *b,
 			      int family)
 {
-	struct rtnl_link_af_ops *af_ops = rtnl_link_af_ops_lookup(family);
+	struct rtnl_link_af_ops *af_ops;
 	int ret = 0;
 
+	if (!a->l_af_data[family] && !b->l_af_data[family])
+		return 0;
+
+	if (!a->l_af_data[family] || !b->l_af_data[family])
+		return ~0;
+
+	af_ops = rtnl_link_af_ops_lookup(family);
 	if (!af_ops)
 		return ~0;
 
-	if (!a->l_af_data[family] && !b->l_af_data[family])
-		goto out;
-
-	if (!a->l_af_data[family] || !b->l_af_data[family]) {
+	if (af_ops->ao_compare == NULL) {
 		ret = ~0;
 		goto out;
 	}
 
-	if (af_ops->ao_compare)
-		ret = af_ops->ao_compare(a, b, family, ~0, 0);
+	ret = af_ops->ao_compare(a, b, family, ~0, 0);
+
 out:
 	rtnl_link_af_ops_put(af_ops);
 
 	return ret;
 }
 
+/**
+ * Compare link info data
+ * @arg a              Link object a
+ * @arg b              Link object b
+ *
+ * This function will compare link_info data between two links
+ * a and b
+ *
+ * @return 0 if link_info data matches or is not present
+ * or != 0 if it mismatches.
+ */
+int rtnl_link_info_data_compare(struct rtnl_link *a, struct rtnl_link *b, int flags)
+{
+	if (a->l_info_ops != b->l_info_ops)
+		return ~0;
+
+	if (!a->l_info_ops || !a->l_info_ops->io_compare)
+		return 0;
+
+	return a->l_info_ops->io_compare(a, b, flags);
+}
+
 /** @} */
 
 /** @} */
