@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/neightbl.c         neighbour tables
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -17,6 +11,7 @@
  */
 
 #include <netlink-private/netlink.h>
+#include <netlink-private/utils.h>
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <netlink/route/rtnl.h>
@@ -24,31 +19,31 @@
 #include <netlink/route/link.h>
 
 /** @cond SKIP */
-#define NEIGHTBL_ATTR_FAMILY       0x001
-#define NEIGHTBL_ATTR_STATS        0x002
-#define NEIGHTBL_ATTR_NAME	  0x004
-#define NEIGHTBL_ATTR_THRESH1	  0x008
-#define NEIGHTBL_ATTR_THRESH2	  0x010
-#define NEIGHTBL_ATTR_THRESH3	  0x020
-#define NEIGHTBL_ATTR_CONFIG	  0x040
-#define NEIGHTBL_ATTR_PARMS	  0x080
-#define NEIGHTBL_ATTR_GC_INTERVAL  0x100
+#define NEIGHTBL_ATTR_FAMILY 0x001
+#define NEIGHTBL_ATTR_STATS 0x002
+#define NEIGHTBL_ATTR_NAME 0x004
+#define NEIGHTBL_ATTR_THRESH1 0x008
+#define NEIGHTBL_ATTR_THRESH2 0x010
+#define NEIGHTBL_ATTR_THRESH3 0x020
+#define NEIGHTBL_ATTR_CONFIG 0x040
+#define NEIGHTBL_ATTR_PARMS 0x080
+#define NEIGHTBL_ATTR_GC_INTERVAL 0x100
 
-#define NEIGHTBLPARM_ATTR_IFINDEX	0x0001
-#define NEIGHTBLPARM_ATTR_REFCNT		0x0002
-#define NEIGHTBLPARM_ATTR_QUEUE_LEN	0x0004
-#define NEIGHTBLPARM_ATTR_APP_PROBES	0x0008
-#define NEIGHTBLPARM_ATTR_UCAST_PROBES	0x0010
-#define NEIGHTBLPARM_ATTR_MCAST_PROBES	0x0020
-#define NEIGHTBLPARM_ATTR_PROXY_QLEN	0x0040
-#define NEIGHTBLPARM_ATTR_REACHABLE_TIME	0x0080
+#define NEIGHTBLPARM_ATTR_IFINDEX 0x0001
+#define NEIGHTBLPARM_ATTR_REFCNT 0x0002
+#define NEIGHTBLPARM_ATTR_QUEUE_LEN 0x0004
+#define NEIGHTBLPARM_ATTR_APP_PROBES 0x0008
+#define NEIGHTBLPARM_ATTR_UCAST_PROBES 0x0010
+#define NEIGHTBLPARM_ATTR_MCAST_PROBES 0x0020
+#define NEIGHTBLPARM_ATTR_PROXY_QLEN 0x0040
+#define NEIGHTBLPARM_ATTR_REACHABLE_TIME 0x0080
 #define NEIGHTBLPARM_ATTR_BASE_REACHABLE_TIME 0x0100
-#define NEIGHTBLPARM_ATTR_RETRANS_TIME	0x0200
-#define NEIGHTBLPARM_ATTR_GC_STALETIME	0x0400
+#define NEIGHTBLPARM_ATTR_RETRANS_TIME 0x0200
+#define NEIGHTBLPARM_ATTR_GC_STALETIME 0x0400
 #define NEIGHTBLPARM_ATTR_DELAY_PROBE_TIME 0x0800
-#define NEIGHTBLPARM_ATTR_ANYCAST_DELAY	0x1000
-#define NEIGHTBLPARM_ATTR_PROXY_DELAY	0x2000
-#define NEIGHTBLPARM_ATTR_LOCKTIME	0x4000
+#define NEIGHTBLPARM_ATTR_ANYCAST_DELAY 0x1000
+#define NEIGHTBLPARM_ATTR_PROXY_DELAY 0x2000
+#define NEIGHTBLPARM_ATTR_LOCKTIME 0x4000
 
 static struct nl_cache_ops rtnl_neightbl_ops;
 static struct nl_object_ops neightbl_obj_ops;
@@ -57,18 +52,18 @@ static struct nl_object_ops neightbl_obj_ops;
 static uint64_t neightbl_compare(struct nl_object *_a, struct nl_object *_b,
 				 uint64_t attrs, int flags)
 {
-	struct rtnl_neightbl *a = (struct rtnl_neightbl *) _a;
-	struct rtnl_neightbl *b = (struct rtnl_neightbl *) _b;
+	struct rtnl_neightbl *a = (struct rtnl_neightbl *)_a;
+	struct rtnl_neightbl *b = (struct rtnl_neightbl *)_b;
 	uint64_t diff = 0;
 
 #define NT_DIFF(ATTR, EXPR) ATTR_DIFF(attrs, NEIGHTBL_ATTR_##ATTR, a, b, EXPR)
 
-	diff |= NT_DIFF(FAMILY,		a->nt_family != b->nt_family);
-	diff |= NT_DIFF(NAME,		strcmp(a->nt_name, b->nt_name));
-	diff |= NT_DIFF(THRESH1,	a->nt_gc_thresh1 != b->nt_gc_thresh1);
-	diff |= NT_DIFF(THRESH2,	a->nt_gc_thresh2 != b->nt_gc_thresh2);
-	diff |= NT_DIFF(THRESH3,	a->nt_gc_thresh3 != b->nt_gc_thresh3);
-	diff |= NT_DIFF(GC_INTERVAL,	a->nt_gc_interval != b->nt_gc_interval);
+	diff |= NT_DIFF(FAMILY, a->nt_family != b->nt_family);
+	diff |= NT_DIFF(NAME, strcmp(a->nt_name, b->nt_name));
+	diff |= NT_DIFF(THRESH1, a->nt_gc_thresh1 != b->nt_gc_thresh1);
+	diff |= NT_DIFF(THRESH2, a->nt_gc_thresh2 != b->nt_gc_thresh2);
+	diff |= NT_DIFF(THRESH3, a->nt_gc_thresh3 != b->nt_gc_thresh3);
+	diff |= NT_DIFF(GC_INTERVAL, a->nt_gc_interval != b->nt_gc_interval);
 
 #undef NT_DIFF
 
@@ -76,8 +71,7 @@ static uint64_t neightbl_compare(struct nl_object *_a, struct nl_object *_b,
 	    !(b->ce_mask & NEIGHTBL_ATTR_PARMS))
 		return diff;
 
-	/* XXX: FIXME: Compare parameter table */
-
+		/* XXX: FIXME: Compare parameter table */
 
 #if 0
 #define REQ(F) (fp->ntp_mask & NEIGHTBLPARM_ATTR_##F)
@@ -105,17 +99,15 @@ static uint64_t neightbl_compare(struct nl_object *_a, struct nl_object *_b,
 	return diff;
 }
 
-
-static struct nla_policy neightbl_policy[NDTA_MAX+1] = {
-	[NDTA_NAME]		= { .type = NLA_STRING,
-				    .maxlen = NTBLNAMSIZ },
-	[NDTA_THRESH1]		= { .type = NLA_U32 },
-	[NDTA_THRESH2]		= { .type = NLA_U32 },
-	[NDTA_THRESH3]		= { .type = NLA_U32 },
-	[NDTA_GC_INTERVAL]	= { .type = NLA_U32 },
-	[NDTA_CONFIG]		= { .minlen = sizeof(struct ndt_config) },
-	[NDTA_STATS]		= { .minlen = sizeof(struct ndt_stats) },
-	[NDTA_PARMS]		= { .type = NLA_NESTED },
+static struct nla_policy neightbl_policy[NDTA_MAX + 1] = {
+	[NDTA_NAME] = { .type = NLA_STRING, .maxlen = NTBLNAMSIZ },
+	[NDTA_THRESH1] = { .type = NLA_U32 },
+	[NDTA_THRESH2] = { .type = NLA_U32 },
+	[NDTA_THRESH3] = { .type = NLA_U32 },
+	[NDTA_GC_INTERVAL] = { .type = NLA_U32 },
+	[NDTA_CONFIG] = { .minlen = sizeof(struct ndt_config) },
+	[NDTA_STATS] = { .minlen = sizeof(struct ndt_stats) },
+	[NDTA_PARMS] = { .type = NLA_NESTED },
 };
 
 static int neightbl_msg_parser(struct nl_cache_ops *ops,
@@ -190,11 +182,11 @@ static int neightbl_msg_parser(struct nl_cache_ops *ops,
 		if (err < 0)
 			goto errout;
 
-#define COPY_ENTRY(name, var) \
-		if (tbp[NDTPA_ ##name]) { \
-			p->ntp_ ##var = nla_get_u32(tbp[NDTPA_ ##name]); \
-			p->ntp_mask |= NEIGHTBLPARM_ATTR_ ##name; \
-		}
+#define COPY_ENTRY(name, var)                                                  \
+	if (tbp[NDTPA_##name]) {                                               \
+		p->ntp_##var = nla_get_u32(tbp[NDTPA_##name]);                 \
+		p->ntp_mask |= NEIGHTBLPARM_ATTR_##name;                       \
+	}
 
 		COPY_ENTRY(IFINDEX, ifindex);
 		COPY_ENTRY(REFCNT, refcnt);
@@ -216,7 +208,7 @@ static int neightbl_msg_parser(struct nl_cache_ops *ops,
 		ntbl->ce_mask |= NEIGHTBL_ATTR_PARMS;
 	}
 
-	err = pp->pp_cb((struct nl_object *) ntbl, pp);
+	err = pp->pp_cb((struct nl_object *)ntbl, pp);
 errout:
 	rtnl_neightbl_put(ntbl);
 	return err;
@@ -227,10 +219,9 @@ static int neightbl_request_update(struct nl_cache *c, struct nl_sock *h)
 	return nl_rtgen_request(h, RTM_GETNEIGHTBL, AF_UNSPEC, NLM_F_DUMP);
 }
 
-
 static void neightbl_dump_line(struct nl_object *arg, struct nl_dump_params *p)
 {
-	struct rtnl_neightbl *ntbl = (struct rtnl_neightbl *) arg;
+	struct rtnl_neightbl *ntbl = (struct rtnl_neightbl *)arg;
 
 	nl_dump_line(p, "%s", ntbl->nt_name);
 
@@ -266,101 +257,102 @@ static void neightbl_dump_line(struct nl_object *arg, struct nl_dump_params *p)
 	nl_dump(p, "\n");
 }
 
-static void neightbl_dump_details(struct nl_object *arg, struct nl_dump_params *p)
+static void neightbl_dump_details(struct nl_object *arg,
+				  struct nl_dump_params *p)
 {
 	char x[32], y[32], z[32];
-	struct rtnl_neightbl *ntbl = (struct rtnl_neightbl *) arg;
+	struct rtnl_neightbl *ntbl = (struct rtnl_neightbl *)arg;
 
 	neightbl_dump_line(arg, p);
 
 	if (ntbl->ce_mask & NEIGHTBL_ATTR_CONFIG) {
 		nl_dump_line(p, "    key-len %u entry-size %u last-flush %s\n",
-			ntbl->nt_config.ndtc_key_len,
-			ntbl->nt_config.ndtc_entry_size,
-			nl_msec2str(ntbl->nt_config.ndtc_last_flush,
-				    x, sizeof(x)));
+			     ntbl->nt_config.ndtc_key_len,
+			     ntbl->nt_config.ndtc_entry_size,
+			     nl_msec2str(ntbl->nt_config.ndtc_last_flush, x,
+					 sizeof(x)));
 
-		nl_dump_line(p, "    gc threshold %u/%u/%u interval %s " \
-				"chain-position %u\n",
-			ntbl->nt_gc_thresh1, ntbl->nt_gc_thresh2,
-			ntbl->nt_gc_thresh3,
-			nl_msec2str(ntbl->nt_gc_interval, x, sizeof(x)),
-			ntbl->nt_config.ndtc_hash_chain_gc);
+		nl_dump_line(p,
+			     "    gc threshold %u/%u/%u interval %s "
+			     "chain-position %u\n",
+			     ntbl->nt_gc_thresh1, ntbl->nt_gc_thresh2,
+			     ntbl->nt_gc_thresh3,
+			     nl_msec2str(ntbl->nt_gc_interval, x, sizeof(x)),
+			     ntbl->nt_config.ndtc_hash_chain_gc);
 
 		nl_dump_line(p, "    hash-rand 0x%08X/0x%08X last-rand %s\n",
-			ntbl->nt_config.ndtc_hash_rnd,
-			ntbl->nt_config.ndtc_hash_mask,
-			nl_msec2str(ntbl->nt_config.ndtc_last_rand,
-				    x, sizeof(x)));
+			     ntbl->nt_config.ndtc_hash_rnd,
+			     ntbl->nt_config.ndtc_hash_mask,
+			     nl_msec2str(ntbl->nt_config.ndtc_last_rand, x,
+					 sizeof(x)));
 	}
 
 	if (ntbl->ce_mask & NEIGHTBL_ATTR_PARMS) {
 		struct rtnl_neightbl_parms *pa = &ntbl->nt_parms;
 
-		nl_dump_line(p, "    refcnt %u pending-queue-limit %u " \
-				"proxy-delayed-queue-limit %u\n",
-			pa->ntp_refcnt,
-			pa->ntp_queue_len,
-			pa->ntp_proxy_qlen);
+		nl_dump_line(p,
+			     "    refcnt %u pending-queue-limit %u "
+			     "proxy-delayed-queue-limit %u\n",
+			     pa->ntp_refcnt, pa->ntp_queue_len,
+			     pa->ntp_proxy_qlen);
 
-		nl_dump_line(p, "    num-userspace-probes %u num-unicast-probes " \
-				"%u num-multicast-probes %u\n",
-			pa->ntp_app_probes,
-			pa->ntp_ucast_probes,
-			pa->ntp_mcast_probes);
+		nl_dump_line(p,
+			     "    num-userspace-probes %u num-unicast-probes "
+			     "%u num-multicast-probes %u\n",
+			     pa->ntp_app_probes, pa->ntp_ucast_probes,
+			     pa->ntp_mcast_probes);
 
-		nl_dump_line(p, "    min-age %s base-reachable-time %s " \
-				"stale-check-interval %s\n",
-			nl_msec2str(pa->ntp_locktime, x, sizeof(x)),
-			nl_msec2str(pa->ntp_base_reachable_time,
-				    y, sizeof(y)),
-			nl_msec2str(pa->ntp_gc_stale_time, z, sizeof(z)));
+		nl_dump_line(p,
+			     "    min-age %s base-reachable-time %s "
+			     "stale-check-interval %s\n",
+			     nl_msec2str(pa->ntp_locktime, x, sizeof(x)),
+			     nl_msec2str(pa->ntp_base_reachable_time, y,
+					 sizeof(y)),
+			     nl_msec2str(pa->ntp_gc_stale_time, z, sizeof(z)));
 
-		nl_dump_line(p, "    initial-probe-delay %s answer-delay %s " \
-				"proxy-answer-delay %s\n",
-			nl_msec2str(pa->ntp_probe_delay, x, sizeof(x)),
-			nl_msec2str(pa->ntp_anycast_delay, y, sizeof(y)),
-			nl_msec2str(pa->ntp_proxy_delay, z, sizeof(z)));
+		nl_dump_line(p,
+			     "    initial-probe-delay %s answer-delay %s "
+			     "proxy-answer-delay %s\n",
+			     nl_msec2str(pa->ntp_probe_delay, x, sizeof(x)),
+			     nl_msec2str(pa->ntp_anycast_delay, y, sizeof(y)),
+			     nl_msec2str(pa->ntp_proxy_delay, z, sizeof(z)));
 	}
 }
 
 static void neightbl_dump_stats(struct nl_object *arg, struct nl_dump_params *p)
 {
-	struct rtnl_neightbl *ntbl = (struct rtnl_neightbl *) arg;
+	struct rtnl_neightbl *ntbl = (struct rtnl_neightbl *)arg;
 
 	neightbl_dump_details(arg, p);
 
 	if (!(ntbl->ce_mask & NEIGHTBL_ATTR_STATS))
 		return;
 
-	nl_dump_line(p, "   " \
-			" lookups %" PRIu64 \
-			" hits %" PRIu64 \
-			" failed %" PRIu64 \
-			" allocations %" PRIu64 \
-			" destroys %" PRIu64 \
-			"\n",
-		ntbl->nt_stats.ndts_lookups,
-		ntbl->nt_stats.ndts_hits,
-		ntbl->nt_stats.ndts_res_failed,
-		ntbl->nt_stats.ndts_allocs,
-		ntbl->nt_stats.ndts_destroys);
+	nl_dump_line(p,
+		     "   "
+		     " lookups %llu hits %llu failed %llu"
+		     " allocations %llu destroys %llu\n",
+		     (long long unsigned)ntbl->nt_stats.ndts_lookups,
+		     (long long unsigned)ntbl->nt_stats.ndts_hits,
+		     (long long unsigned)ntbl->nt_stats.ndts_res_failed,
+		     (long long unsigned)ntbl->nt_stats.ndts_allocs,
+		     (long long unsigned)ntbl->nt_stats.ndts_destroys);
 
-	nl_dump_line(p, "   " \
-			" hash-grows %" PRIu64 \
-			" forced-gc-runs %" PRIu64 \
-			" periodic-gc-runs %" PRIu64 \
-			"\n",
-		ntbl->nt_stats.ndts_hash_grows,
-		ntbl->nt_stats.ndts_forced_gc_runs,
-		ntbl->nt_stats.ndts_periodic_gc_runs);
+	nl_dump_line(p,
+		     "   "
+		     " hash-grows %llu forced-gc-runs %llu"
+		     " periodic-gc-runs %llu\n",
+		     (long long unsigned)ntbl->nt_stats.ndts_hash_grows,
+		     (long long unsigned)ntbl->nt_stats.ndts_forced_gc_runs,
+		     (long long unsigned)ntbl->nt_stats.ndts_periodic_gc_runs);
 
-	nl_dump_line(p, "   " \
-			" rcv-unicast-probes %" PRIu64 \
-			" rcv-multicast-probes %" PRIu64 \
-			"\n",
-		ntbl->nt_stats.ndts_rcv_probes_ucast,
-		ntbl->nt_stats.ndts_rcv_probes_mcast);
+	nl_dump_line(p,
+		     "   "
+		     " rcv-unicast-probes %llu"
+		     " rcv-multicast-probes %llu"
+		     "\n",
+		     (long long unsigned)ntbl->nt_stats.ndts_rcv_probes_ucast,
+		     (long long unsigned)ntbl->nt_stats.ndts_rcv_probes_mcast);
 }
 
 /**
@@ -370,12 +362,12 @@ static void neightbl_dump_stats(struct nl_object *arg, struct nl_dump_params *p)
 
 struct rtnl_neightbl *rtnl_neightbl_alloc(void)
 {
-	return (struct rtnl_neightbl *) nl_object_alloc(&neightbl_obj_ops);
+	return (struct rtnl_neightbl *)nl_object_alloc(&neightbl_obj_ops);
 }
 
 void rtnl_neightbl_put(struct rtnl_neightbl *neightbl)
 {
-	nl_object_put((struct nl_object *) neightbl);
+	nl_object_put((struct nl_object *)neightbl);
 }
 
 /** @} */
@@ -422,11 +414,11 @@ struct rtnl_neightbl *rtnl_neightbl_get(struct nl_cache *cache,
 	if (cache->c_ops != &rtnl_neightbl_ops)
 		return NULL;
 
-	nl_list_for_each_entry(nt, &cache->c_items, ce_list) {
+	nl_list_for_each_entry (nt, &cache->c_items, ce_list) {
 		if (!strcasecmp(nt->nt_name, name) &&
 		    ((!ifindex && !nt->nt_parms.ntp_ifindex) ||
 		     (ifindex && ifindex == nt->nt_parms.ntp_ifindex))) {
-			nl_object_get((struct nl_object *) nt);
+			nl_object_get((struct nl_object *)nt);
 			return nt;
 		}
 	}
@@ -485,8 +477,7 @@ int rtnl_neightbl_build_change_request(struct rtnl_neightbl *old,
 		NLA_PUT_U32(m, NDTA_THRESH2, tmpl->nt_gc_thresh2);
 
 	if (tmpl->ce_mask & NEIGHTBL_ATTR_GC_INTERVAL)
-		NLA_PUT_U64(m, NDTA_GC_INTERVAL,
-				      tmpl->nt_gc_interval);
+		NLA_PUT_U64(m, NDTA_GC_INTERVAL, tmpl->nt_gc_interval);
 
 	if (tmpl->ce_mask & NEIGHTBL_ATTR_PARMS) {
 		struct rtnl_neightbl_parms *p = &tmpl->nt_parms;
@@ -497,8 +488,7 @@ int rtnl_neightbl_build_change_request(struct rtnl_neightbl *old,
 
 		if (old->nt_parms.ntp_mask & NEIGHTBLPARM_ATTR_IFINDEX)
 			NLA_PUT_U32(parms, NDTPA_IFINDEX,
-					      old->nt_parms.ntp_ifindex);
-
+				    old->nt_parms.ntp_ifindex);
 
 		if (p->ntp_mask & NEIGHTBLPARM_ATTR_QUEUE_LEN)
 			NLA_PUT_U32(parms, NDTPA_QUEUE_LEN, p->ntp_queue_len);
@@ -515,8 +505,7 @@ int rtnl_neightbl_build_change_request(struct rtnl_neightbl *old,
 				    p->ntp_mcast_probes);
 
 		if (p->ntp_mask & NEIGHTBLPARM_ATTR_PROXY_QLEN)
-			NLA_PUT_U32(parms, NDTPA_PROXY_QLEN,
-				    p->ntp_proxy_qlen);
+			NLA_PUT_U32(parms, NDTPA_PROXY_QLEN, p->ntp_proxy_qlen);
 
 		if (p->ntp_mask & NEIGHTBLPARM_ATTR_BASE_REACHABLE_TIME)
 			NLA_PUT_U64(parms, NDTPA_BASE_REACHABLE_TIME,
@@ -540,7 +529,7 @@ int rtnl_neightbl_build_change_request(struct rtnl_neightbl *old,
 
 		if (p->ntp_mask & NEIGHTBLPARM_ATTR_PROXY_DELAY)
 			NLA_PUT_U64(parms, NDTPA_PROXY_DELAY,
-					      p->ntp_proxy_delay);
+				    p->ntp_proxy_delay);
 
 		if (p->ntp_mask & NEIGHTBLPARM_ATTR_LOCKTIME)
 			NLA_PUT_U64(parms, NDTPA_LOCKTIME, p->ntp_locktime);
@@ -579,7 +568,7 @@ int rtnl_neightbl_change(struct nl_sock *sk, struct rtnl_neightbl *old,
 {
 	struct nl_msg *msg;
 	int err;
-	
+
 	if ((err = rtnl_neightbl_build_change_request(old, tmpl, &msg)) < 0)
 		return err;
 
@@ -630,7 +619,7 @@ void rtnl_neightbl_set_gc_tresh3(struct rtnl_neightbl *ntbl, int thresh)
 
 void rtnl_neightbl_set_name(struct rtnl_neightbl *ntbl, const char *name)
 {
-	strncpy(ntbl->nt_name, name, sizeof(ntbl->nt_name) - 1);
+	_nl_strncpy_trunc(ntbl->nt_name, name, sizeof(ntbl->nt_name));
 	ntbl->ce_mask |= NEIGHTBL_ATTR_NAME;
 }
 
