@@ -1,12 +1,5 @@
 /* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/route.c	Routes
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -18,6 +11,7 @@
  */
 
 #include <netlink-private/netlink.h>
+#include <netlink-private/nl-auto.h>
 #include <netlink/netlink.h>
 #include <netlink/cache.h>
 #include <netlink/utils.h>
@@ -131,6 +125,32 @@ int rtnl_route_build_add_request(struct rtnl_route *tmpl, int flags,
 			       result);
 }
 
+int rtnl_route_lookup(struct nl_sock *sk, struct nl_addr *dst,
+                      struct rtnl_route **result)
+{
+	_nl_auto_nl_msg struct nl_msg *msg = NULL;
+	_nl_auto_rtnl_route struct rtnl_route *tmpl = NULL;
+	struct nl_object *obj;
+	int err;
+
+	tmpl = rtnl_route_alloc();
+	rtnl_route_set_dst(tmpl, dst);
+	err = build_route_msg(tmpl, RTM_GETROUTE, 0, &msg);
+	if (err < 0)
+		return err;
+
+	err = nl_send_auto(sk, msg);
+	if (err < 0)
+		return err;
+
+	if ((err = nl_pickup(sk, route_msg_parser, &obj)) < 0)
+		return err;
+
+	*result = (struct rtnl_route *)obj;
+	wait_for_ack(sk);
+	return 0;
+}
+
 int rtnl_route_add(struct nl_sock *sk, struct rtnl_route *route, int flags)
 {
 	struct nl_msg *msg;
