@@ -1,12 +1,5 @@
 /* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/route/rule.c          Routing Rules
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2010 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -66,6 +59,9 @@ static int rule_clone(struct nl_object *_dst, struct nl_object *_src)
 	struct rtnl_rule *dst = nl_object_priv(_dst);
 	struct rtnl_rule *src = nl_object_priv(_src);
 
+	dst->r_src = NULL;
+	dst->r_dst = NULL;
+
 	if (src->r_src)
 		if (!(dst->r_src = nl_addr_clone(src->r_src)))
 			return -NLE_NOMEM;
